

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF
GO

DROP PROCEDURE dbo.Pron_IngHE_Reserva
GO

CREATE PROCEDURE dbo.Pron_IngHE_Reserva
	   @dia_inic DATETIME, 
	   @dia_fin DATETIME, 
	   @dia_reserv DATETIME, 
	   @mone INT, 
	   @tasa_ext NUMERIC(13, 8)
AS

/*   Calcula ingreso de Hospedaje en RESERVAS, dentro un rango de fechas a base de la fecha de reservacion hacia el pasado */

BEGIN

	DECLARE @Ing_Local NUMERIC(15, 2), 
	        @Ing_LocalII NUMERIC(15, 2), 
			@mese VARCHAR(10), 
			@ciclo DATETIME, 
			@ing_reserva NUMERIC(15, 2)
	DECLARE @Tab_PREH TABLE
	(
		id_rese INT, 
		id_cont INT, 
		id_estado INT, 
		id_mone INT, 
		tasa NUMERIC(13, 8), 
		fecha_ent DATETIME, 
		fecha_sal DATETIME, 
		fecha_reserv DATETIME, 
		cant_habi INT, 
		precio_hab MONEY, 
		precio NUMERIC(18, 2), 
		cantidad TINYINT, 
		nombre VARCHAR(40), 
		nomb_reserv VARCHAR(40), 
		nro_reserva INT, 
		fecha_anulada DATETIME, 
		id_lres INT, 
		id_ralo INT, 
		id_agen INT, 
		tasa_fija NUMERIC(13, 8), 
		cod_mone CHAR(3)
	)
     
	/* CREA TABLA DE POSIBLES DATOS Pasado--Futuro */

	INSERT INTO @Tab_PREH
	SELECT rese.id_rese, 
		   rese.id_cont, 
		   id_estado, 
		   cont.id_mone, 
		   tasa, 
		   fecha_ent = reau.fecha_desde, 
		   fecha_sal = reau.fecha_hasta, 
		   fecha_reserv, 
		   cant_habi, 
		   precio_hab, 
		   precio, 
		   cantidad, 
		   nombre, 
		   nomb_reserv, 
		   nro_reserva, 
		   fecha_anulada, 
		   lres.id_lres, 
		   lres.id_ralo, 
		   id_agen, 
		   tasa_fija, 
		   cod_mone
	FROM       hoterese rese
	INNER JOIN Hotelres lres ON rese.id_rese = lres.id_rese
	INNER JOIN Hotecont cont ON rese.id_cont = cont.id_cont
	INNER JOIN Hotereau reau ON lres.id_lres = reau.id_lres
	INNER JOIN Siclamon sicla ON cont.id_mone = sicla.id_mone
	WHERE rese.id_estado IN(2, 3, 5)
	  AND reau.id_con0 IN(SELECT thab.id_con0 FROM hotethab thab)
	UNION
	SELECT rese.id_rese, 
		   rese.id_cont, 
		   id_estado, 
		   cont.id_mone, 
		   tasa, 
		   fecha_ent = reau.fecha_desde, 
		   fecha_sal = reau.fecha_hasta, 
		   fecha_reserv, 
		   cant_habi, 
		   precio_hab, 
		   precio, 
		   cantidad, 
		   nombre, 
		   nomb_reserv, 
		   nro_reserva, 
		   fecha_anulada, 
		   lres.id_lres, 
		   lres.id_ralo, 
		   id_agen, 
		   tasa_fija, 
		   cod_mone
	FROM       HOTEHRES rese
	INNER JOIN HOTEHLRE lres ON rese.id_rese = lres.id_rese
	INNER JOIN Hotecont cont ON rese.id_cont = cont.id_cont
	INNER JOIN HOTEHREAU reau ON lres.id_lres = reau.id_lres
	INNER JOIN Siclamon sicla ON cont.id_mone = sicla.id_mone
	WHERE rese.id_estado IN(2, 3, 5)
	  AND reau.id_con0 IN(SELECT thab.id_con0 FROM hotethab thab)
	ORDER BY fecha_reserv

	/* CREA CONSULTA DATOS DE INGRESO DE HOSPEDAJE POR MES */

	SET @mese = LEFT(YEAR(@dia_inic), 4) + RIGHT('0' + LEFT(MONTH(@dia_inic), 2), 2)

	WHILE @mese <= LEFT(YEAR(@dia_fin), 4) + RIGHT('0' + LEFT(MONTH(@dia_fin), 2), 2)
		BEGIN 
 
			SELECT @ciclo = DATEADD(s,-1,DATEADD(mm, DATEDIFF(m,0,@dia_inic)+1,0)) -- Obtener ltimo da del mes en base a la fecha dada.

			SET @ing_reserva = 0
			WHILE @dia_inic <= @ciclo
				BEGIN
				
					DELETE FROM @Tab_PREH
					WHERE id_estado = 2 
					  AND id_rese  IN  (SELECT ISNULL(face.id_rese, 0)
										FROM       Hoteface face
										INNER JOIN Hotefact fact ON face.id_face = fact.id_face
										INNER JOIN Hotehabi habi ON face.id_habi = habi.id_habi
										INNER JOIN Hotethab thab ON habi.id_thab = thab.id_thab
										INNER JOIN Hotepre0 pre0 ON face.id_mone = pre0.id_mone AND thab.id_con0 = pre0.id_con0
										INNER JOIN Hotelinc linc ON fact.id_fact = linc.id_fact AND pre0.id_pre0 = linc.id_pre0
										INNER JOIN Siclamon sicla ON face.id_mone = sicla.id_mone
										WHERE DATEDIFF(day, linc.fecha_desde, @dia_inic) >= 0
										  AND DATEDIFF(day, @dia_inic, linc.fecha_hasta) > 0
										   OR DATEDIFF(day, linc.fecha_desde, @dia_inic) >= 0
										  AND DATEDIFF(day, @dia_inic, linc.fecha_hasta) > 0)

					SELECT @Ing_Local = COALESCE(@Ing_Local, 0) + 
					                    COALESCE(SUM(precio * cantidad * CASE WHEN id_mone != @mone THEN @tasa_ext / COALESCE(tasa_fija, tasa)
																			  ELSE 1
																			  END), 0.00)
					FROM @Tab_PREH
					WHERE DATEDIFF(DAY, @dia_reserv, fecha_reserv) <= 0
					  AND DATEDIFF(day, fecha_ent, @dia_inic) >= 0
					  AND DATEDIFF(day, @dia_inic, fecha_sal) > 0
					  
					-- Porque al parecer Front lo hace, lo vamos a hacer tambin
					SELECT @Ing_LocalII = SUM(cantidad_linc * CONVERT(NUMERIC(15, 2), precio_linc) * CASE WHEN face1.id_mone != @mone THEN @tasa_ext / sicla.tasa
																										  ELSE 1
																									      END)
					FROM(
						SELECT fact.id_fact, 
							   pre0.id_pre0, 
							   face.id_mone
						FROM       Hoteface face
						INNER JOIN Hotefact fact ON face.id_face = fact.id_face
						INNER JOIN Hotehabi habi ON face.id_habi = habi.id_habi
						INNER JOIN Hotethab thab ON habi.id_thab = thab.id_thab
						INNER JOIN Hotepre0 pre0 ON face.id_mone = pre0.id_mone AND thab.id_con0 = pre0.id_con0) face1
						INNER JOIN Hotelinc linc ON face1.id_fact = linc.id_fact AND face1.id_pre0 = linc.id_pre0
						INNER JOIN Siclamon sicla ON face1.id_mone = sicla.id_mone
						WHERE DATEDIFF(day, linc.fecha_desde, @dia_inic) >= 0
						  AND DATEDIFF(day, @dia_inic, linc.fecha_hasta) > 0
						   OR DATEDIFF(day, linc.fecha_desde, @dia_inic) >= 0
						  AND DATEDIFF(day, @dia_inic, linc.fecha_hasta) > 0
					  
					/*SET @Ing_LocalII = COALESCE(@Ing_LocalII, 0) 
						+ COALESCE((SELECT SUM(cantidad_linc * CONVERT(NUMERIC(15, 2), precio_linc) * CASE WHEN face1.id_mone != @mone THEN @tasa_ext / sicla.tasa
																										  ELSE 1
																									      END)
									FROM(
										SELECT fact.id_hfac, 
											   pre0.id_pre0, 
											   face.id_mone
										FROM       Hotehhab face
										INNER JOIN Hotehfac fact ON face.id_face = fact.id_face
										INNER JOIN Hotehabi habi ON face.id_habi = habi.id_habi
										INNER JOIN Hotethab thab ON habi.id_thab = thab.id_thab
										INNER JOIN Hotepre0 pre0 ON face.id_mone = pre0.id_mone AND thab.id_con0 = pre0.id_con0) face1
										INNER JOIN Hotelinc linc ON face1.id_fact = linc.id_fact AND face1.id_pre0 = linc.id_pre0 -- Histrico ??
										INNER JOIN Siclamon sicla ON face1.id_mone = sicla.id_mone
										WHERE DATEDIFF(day, linc.fecha_desde, @dia_inic) >= 0
										  AND DATEDIFF(day, @dia_inic, linc.fecha_hasta) > 0
										   OR DATEDIFF(day, linc.fecha_desde, @dia_inic) >= 0
										  AND DATEDIFF(day, @dia_inic, linc.fecha_hasta) > 0), 0)*/
					  
					UPDATE DEBGDAIPG
					SET ing_Reserva = ing_Reserva + COALESCE(@Ing_Local, 0) + COALESCE(@Ing_LocalII, 0)
					WHERE fecha = CAST(@dia_inic AS DATE)

					SET @dia_inic = @dia_inic + 1
					SET @ing_reserva = @ing_reserva + COALESCE(@Ing_Local, 0) + COALESCE(@Ing_LocalII, 0)
					SET @Ing_Local = 0
				END   	
		
			/* CREA TABLA CON DATOS DE INGRESO DE HOSPEDAJE POR MES */

			UPDATE HOTEDAIPG
			SET ing_Reserva = Ing_Reserva + @ing_reserva
			WHERE mes = MONTH(@ciclo)
			  AND fyear = YEAR(@ciclo)

			SET @mese = LEFT(YEAR(@dia_inic), 4) + RIGHT('0' + LEFT(MONTH(@dia_inic), 2), 2)

		END
END



